import { jsxs, jsx, Fragment } from 'react/jsx-runtime';
import { useLexicalComposerContext } from '@lexical/react/LexicalComposerContext';
import React, { useRef, useState, useCallback, useEffect, useMemo } from 'react';
import { $getSelection, $isRangeSelection, SELECTION_CHANGE_COMMAND, CAN_UNDO_COMMAND, CAN_REDO_COMMAND, $getNodeByKey, UNDO_COMMAND, REDO_COMMAND, FORMAT_TEXT_COMMAND, FORMAT_ELEMENT_COMMAND, $createParagraphNode } from 'lexical';
import { L as LexicalLink_1 } from '../../node_modules/@lexical/link/LexicalLink.js';
import { L as LexicalSelection_1 } from '../../node_modules/@lexical/selection/LexicalSelection.js';
import { L as LexicalUtils_1 } from '../../node_modules/@lexical/utils/LexicalUtils.js';
import { L as LexicalList_1 } from '../../node_modules/@lexical/list/LexicalList.js';
import { createPortal } from 'react-dom';
import { L as LexicalRichText_1 } from '../../node_modules/@lexical/rich-text/LexicalRichText.js';
import { L as LexicalCode_1 } from '../../node_modules/@lexical/code/LexicalCode.js';
import { INSERT_IMAGE_COMMAND } from '../nodes/image.js';
import { INSERT_VIDEO_COMMAND } from '../nodes/video.js';
import { Modal } from '@cpchain-foundation/ui';
import { ImageSelection } from '../components/img-selection.js';
import { AddUrl } from '../components/add-url.js';

const LowPriority = 1;
const supportedBlockTypes = new Set([
    "paragraph",
    "quote",
    "code",
    "h1",
    "h2",
    "ul",
    "ol"
]);
const blockTypeToBlockName = {
    code: "Code Block",
    h1: "Large Heading",
    h2: "Small Heading",
    h3: "Heading",
    h4: "Heading",
    h5: "Heading",
    ol: "Numbered List",
    paragraph: "Normal",
    quote: "Quote",
    ul: "Bulleted List"
};
function Divider() {
    return jsx("div", { className: "divider" });
}
function positionEditorElement(editor, rect) {
    if (rect === null) {
        editor.style.opacity = "0";
        editor.style.top = "-1000px";
        editor.style.left = "-1000px";
    }
    else {
        editor.style.opacity = "1";
        editor.style.top = `${rect.top + rect.height + window.pageYOffset + 10}px`;
        editor.style.left = `${rect.left + window.pageXOffset - editor.offsetWidth / 2 + rect.width / 2}px`;
    }
}
function FloatingLinkEditor({ editor }) {
    const editorRef = useRef(null);
    const inputRef = useRef(null);
    const mouseDownRef = useRef(false);
    const [linkUrl, setLinkUrl] = useState("");
    const [isEditMode, setEditMode] = useState(false);
    const [lastSelection, setLastSelection] = useState(null);
    const updateLinkEditor = useCallback(() => {
        const selection = $getSelection();
        if ($isRangeSelection(selection)) {
            const node = getSelectedNode(selection);
            const parent = node.getParent();
            if (LexicalLink_1.$isLinkNode(parent)) {
                setLinkUrl(parent.getURL());
            }
            else if (LexicalLink_1.$isLinkNode(node)) {
                setLinkUrl(node.getURL());
            }
            else {
                setLinkUrl("");
            }
        }
        const editorElem = editorRef.current;
        const nativeSelection = window.getSelection();
        const activeElement = document.activeElement;
        if (editorElem === null) {
            return;
        }
        const rootElement = editor.getRootElement();
        if (nativeSelection &&
            selection !== null &&
            !nativeSelection.isCollapsed &&
            rootElement !== null &&
            rootElement.contains(nativeSelection.anchorNode)) {
            const domRange = nativeSelection.getRangeAt(0);
            let rect;
            if (nativeSelection.anchorNode === rootElement) {
                let inner = rootElement;
                while (inner.firstElementChild != null) {
                    inner = inner.firstElementChild;
                }
                rect = inner.getBoundingClientRect();
            }
            else {
                rect = domRange.getBoundingClientRect();
            }
            if (!mouseDownRef.current) {
                positionEditorElement(editorElem, rect);
            }
            setLastSelection(selection);
        }
        else if (!activeElement || activeElement.className !== "link-input") {
            positionEditorElement(editorElem, null);
            setLastSelection(null);
            setEditMode(false);
            setLinkUrl("");
        }
        return true;
    }, [editor]);
    useEffect(() => {
        return LexicalUtils_1.mergeRegister(editor.registerUpdateListener(({ editorState }) => {
            editorState.read(() => {
                updateLinkEditor();
            });
        }), editor.registerCommand(SELECTION_CHANGE_COMMAND, () => {
            updateLinkEditor();
            return true;
        }, LowPriority));
    }, [editor, updateLinkEditor]);
    useEffect(() => {
        editor.getEditorState().read(() => {
            updateLinkEditor();
        });
    }, [editor, updateLinkEditor]);
    useEffect(() => {
        if (isEditMode && inputRef.current) {
            inputRef.current.focus();
        }
    }, [isEditMode]);
    return (jsx("div", { ref: editorRef, className: "link-editor", children: isEditMode ? (jsx("input", { ref: inputRef, className: "link-input", value: linkUrl, onChange: (event) => {
                setLinkUrl(event.target.value);
            }, onKeyDown: (event) => {
                if (event.key === "Enter") {
                    event.preventDefault();
                    if (lastSelection !== null) {
                        if (linkUrl !== "") {
                            editor.dispatchCommand(LexicalLink_1.TOGGLE_LINK_COMMAND, linkUrl);
                        }
                        setEditMode(false);
                    }
                }
                else if (event.key === "Escape") {
                    event.preventDefault();
                    setEditMode(false);
                }
            } })) : (jsx(Fragment, { children: jsxs("div", { className: "link-input", children: [jsx("a", { href: linkUrl, target: "_blank", rel: "noopener noreferrer", children: linkUrl }), jsx("div", { className: "link-edit", role: "button", tabIndex: 0, onMouseDown: (event) => event.preventDefault(), onClick: () => {
                            setEditMode(true);
                        } })] }) })) }));
}
function Select({ onChange, className, options, value }) {
    return (jsxs("select", { className: className, onChange: onChange, value: value, children: [jsx("option", { hidden: true, value: "" }), options.map((option) => (jsx("option", { value: option, children: option }, option)))] }));
}
function getSelectedNode(selection) {
    const anchor = selection.anchor;
    const focus = selection.focus;
    const anchorNode = selection.anchor.getNode();
    const focusNode = selection.focus.getNode();
    if (anchorNode === focusNode) {
        return anchorNode;
    }
    const isBackward = selection.isBackward();
    if (isBackward) {
        return LexicalSelection_1.$isAtNodeEnd(focus) ? anchorNode : focusNode;
    }
    else {
        return LexicalSelection_1.$isAtNodeEnd(anchor) ? focusNode : anchorNode;
    }
}
function BlockOptionsDropdownList({ editor, blockType, toolbarRef, setShowBlockOptionsDropDown }) {
    const dropDownRef = useRef(null);
    useEffect(() => {
        const toolbar = toolbarRef.current;
        const dropDown = dropDownRef.current;
        if (toolbar !== null && dropDown !== null) {
            const { top, left } = toolbar.getBoundingClientRect();
            dropDown.style.top = `${top + 40}px`;
            dropDown.style.left = `${left}px`;
        }
    }, [dropDownRef, toolbarRef]);
    useEffect(() => {
        const dropDown = dropDownRef.current;
        const toolbar = toolbarRef.current;
        if (dropDown !== null && toolbar !== null) {
            const handle = (event) => {
                const target = event.target;
                if (!dropDown.contains(target) && !toolbar.contains(target)) {
                    setShowBlockOptionsDropDown(false);
                }
            };
            document.addEventListener("click", handle);
            return () => {
                document.removeEventListener("click", handle);
            };
        }
    }, [dropDownRef, setShowBlockOptionsDropDown, toolbarRef]);
    const formatParagraph = () => {
        if (blockType !== "paragraph") {
            editor.update(() => {
                const selection = $getSelection();
                if ($isRangeSelection(selection)) {
                    LexicalSelection_1.$wrapNodes(selection, () => $createParagraphNode());
                }
            });
        }
        setShowBlockOptionsDropDown(false);
    };
    const formatLargeHeading = () => {
        if (blockType !== "h1") {
            editor.update(() => {
                const selection = $getSelection();
                if ($isRangeSelection(selection)) {
                    LexicalSelection_1.$wrapNodes(selection, () => LexicalRichText_1.$createHeadingNode("h1"));
                }
            });
        }
        setShowBlockOptionsDropDown(false);
    };
    const formatSmallHeading = () => {
        if (blockType !== "h2") {
            editor.update(() => {
                const selection = $getSelection();
                if ($isRangeSelection(selection)) {
                    LexicalSelection_1.$wrapNodes(selection, () => LexicalRichText_1.$createHeadingNode("h2"));
                }
            });
        }
        setShowBlockOptionsDropDown(false);
    };
    const formatBulletList = () => {
        if (blockType !== "ul") {
            editor.dispatchCommand(LexicalList_1.INSERT_UNORDERED_LIST_COMMAND);
        }
        else {
            editor.dispatchCommand(LexicalList_1.REMOVE_LIST_COMMAND);
        }
        setShowBlockOptionsDropDown(false);
    };
    const formatNumberedList = () => {
        if (blockType !== "ol") {
            editor.dispatchCommand(LexicalList_1.INSERT_ORDERED_LIST_COMMAND);
        }
        else {
            editor.dispatchCommand(LexicalList_1.REMOVE_LIST_COMMAND);
        }
        setShowBlockOptionsDropDown(false);
    };
    const formatQuote = () => {
        if (blockType !== "quote") {
            editor.update(() => {
                const selection = $getSelection();
                if ($isRangeSelection(selection)) {
                    LexicalSelection_1.$wrapNodes(selection, () => LexicalRichText_1.$createQuoteNode());
                }
            });
        }
        setShowBlockOptionsDropDown(false);
    };
    const formatCode = () => {
        if (blockType !== "code") {
            editor.update(() => {
                const selection = $getSelection();
                if ($isRangeSelection(selection)) {
                    LexicalSelection_1.$wrapNodes(selection, () => LexicalCode_1.$createCodeNode());
                }
            });
        }
        setShowBlockOptionsDropDown(false);
    };
    return (jsxs("div", { className: "dropdown", ref: dropDownRef, children: [jsxs("button", { className: "item", onClick: formatParagraph, children: [jsx("span", { className: "icon paragraph" }), jsx("span", { className: "text", children: "Normal" }), blockType === "paragraph" && jsx("span", { className: "active" })] }), jsxs("button", { className: "item", onClick: formatLargeHeading, children: [jsx("span", { className: "icon large-heading" }), jsx("span", { className: "text", children: "Large Heading" }), blockType === "h1" && jsx("span", { className: "active" })] }), jsxs("button", { className: "item", onClick: formatSmallHeading, children: [jsx("span", { className: "icon small-heading" }), jsx("span", { className: "text", children: "Small Heading" }), blockType === "h2" && jsx("span", { className: "active" })] }), jsxs("button", { className: "item", onClick: formatBulletList, children: [jsx("span", { className: "icon bullet-list" }), jsx("span", { className: "text", children: "Bullet List" }), blockType === "ul" && jsx("span", { className: "active" })] }), jsxs("button", { className: "item", onClick: formatNumberedList, children: [jsx("span", { className: "icon numbered-list" }), jsx("span", { className: "text", children: "Numbered List" }), blockType === "ol" && jsx("span", { className: "active" })] }), jsxs("button", { className: "item", onClick: formatQuote, children: [jsx("span", { className: "icon quote" }), jsx("span", { className: "text", children: "Quote" }), blockType === "quote" && jsx("span", { className: "active" })] }), jsxs("button", { className: "item", onClick: formatCode, children: [jsx("span", { className: "icon code" }), jsx("span", { className: "text", children: "Code Block" }), blockType === "code" && jsx("span", { className: "active" })] })] }));
}
function ToolbarPlugin({ uploadImageUrl: uploadUrl, getImages }) {
    const [editor] = useLexicalComposerContext();
    const toolbarRef = useRef(null);
    const [canUndo, setCanUndo] = useState(false);
    const [canRedo, setCanRedo] = useState(false);
    const [blockType, setBlockType] = useState("paragraph");
    const [selectedElementKey, setSelectedElementKey] = useState(null);
    const [showBlockOptionsDropDown, setShowBlockOptionsDropDown] = useState(false);
    const [codeLanguage, setCodeLanguage] = useState("");
    const [isRTL, setIsRTL] = useState(false);
    const [isLink, setIsLink] = useState(false);
    const [isBold, setIsBold] = useState(false);
    const [isItalic, setIsItalic] = useState(false);
    const [isUnderline, setIsUnderline] = useState(false);
    const [isStrikethrough, setIsStrikethrough] = useState(false);
    const [isCode, setIsCode] = useState(false);
    const updateToolbar = useCallback(() => {
        const selection = $getSelection();
        if ($isRangeSelection(selection)) {
            const anchorNode = selection.anchor.getNode();
            const element = anchorNode.getKey() === "root"
                ? anchorNode
                : anchorNode.getTopLevelElementOrThrow();
            const elementKey = element.getKey();
            const elementDOM = editor.getElementByKey(elementKey);
            if (elementDOM !== null) {
                setSelectedElementKey(elementKey);
                if (LexicalList_1.$isListNode(element)) {
                    const parentList = LexicalUtils_1.$getNearestNodeOfType(anchorNode, LexicalList_1.ListNode);
                    const type = parentList ? parentList.getTag() : element.getTag();
                    setBlockType(type);
                }
                else {
                    const type = LexicalRichText_1.$isHeadingNode(element)
                        ? element.getTag()
                        : element.getType();
                    setBlockType(type);
                    if (LexicalCode_1.$isCodeNode(element)) {
                        setCodeLanguage(element.getLanguage() || LexicalCode_1.getDefaultCodeLanguage());
                    }
                }
            }
            // Update text format
            setIsBold(selection.hasFormat("bold"));
            setIsItalic(selection.hasFormat("italic"));
            setIsUnderline(selection.hasFormat("underline"));
            setIsStrikethrough(selection.hasFormat("strikethrough"));
            setIsCode(selection.hasFormat("code"));
            setIsRTL(LexicalSelection_1.$isParentElementRTL(selection));
            // Update links
            const node = getSelectedNode(selection);
            const parent = node.getParent();
            if (LexicalLink_1.$isLinkNode(parent) || LexicalLink_1.$isLinkNode(node)) {
                setIsLink(true);
            }
            else {
                setIsLink(false);
            }
        }
    }, [editor]);
    useEffect(() => {
        return LexicalUtils_1.mergeRegister(editor.registerUpdateListener(({ editorState }) => {
            editorState.read(() => {
                updateToolbar();
            });
        }), editor.registerCommand(SELECTION_CHANGE_COMMAND, (_payload, newEditor) => {
            updateToolbar();
            return false;
        }, LowPriority), editor.registerCommand(CAN_UNDO_COMMAND, (payload) => {
            setCanUndo(payload);
            return false;
        }, LowPriority), editor.registerCommand(CAN_REDO_COMMAND, (payload) => {
            setCanRedo(payload);
            return false;
        }, LowPriority));
    }, [editor, updateToolbar]);
    const [video, setVideoActive] = React.useState(false);
    const [imgActive, setImgActive] = React.useState(false);
    const codeLanguges = useMemo(() => LexicalCode_1.getCodeLanguages(), []);
    const onCodeLanguageSelect = useCallback((e) => {
        editor.update(() => {
            if (selectedElementKey !== null) {
                const node = $getNodeByKey(selectedElementKey);
                if (LexicalCode_1.$isCodeNode(node)) {
                    node.setLanguage(e.target.value);
                }
            }
        });
    }, [editor, selectedElementKey]);
    const insertLink = useCallback(() => {
        if (!isLink) {
            editor.dispatchCommand(LexicalLink_1.TOGGLE_LINK_COMMAND, "https://");
        }
        else {
            editor.dispatchCommand(LexicalLink_1.TOGGLE_LINK_COMMAND, null);
        }
    }, [editor, isLink]);
    const insertImage = useCallback((url) => {
        // Executing command defined in a plugin
        editor.dispatchCommand(INSERT_IMAGE_COMMAND, url);
    }, [editor]);
    const insertVideo = (url) => {
        console.log(insertVideo, url);
        // Executing command defined in a plugin
        editor.dispatchCommand(INSERT_VIDEO_COMMAND, url);
    };
    return (jsxs("div", { className: "toolbar", ref: toolbarRef, children: [jsx("button", { disabled: !canUndo, onClick: () => {
                    editor.dispatchCommand(UNDO_COMMAND);
                }, className: "toolbar-item spaced", "aria-label": "Undo", children: jsx("i", { className: "format undo" }) }), jsx("button", { disabled: !canRedo, onClick: () => {
                    editor.dispatchCommand(REDO_COMMAND);
                }, className: "toolbar-item", "aria-label": "Redo", children: jsx("i", { className: "format redo" }) }), jsx(Divider, {}), supportedBlockTypes.has(blockType) && (jsxs(Fragment, { children: [jsxs("button", { className: "toolbar-item block-controls", onClick: () => setShowBlockOptionsDropDown(!showBlockOptionsDropDown), "aria-label": "Formatting Options", children: [jsx("span", { className: "icon block-type " + blockType }), jsx("span", { className: "text", children: blockTypeToBlockName[blockType] }), jsx("i", { className: "chevron-down" })] }), showBlockOptionsDropDown &&
                        createPortal(jsx(BlockOptionsDropdownList, { editor: editor, blockType: blockType, toolbarRef: toolbarRef, setShowBlockOptionsDropDown: setShowBlockOptionsDropDown }), document.body), jsx(Divider, {})] })), blockType === "code" ? (jsxs(Fragment, { children: [jsx(Select, { className: "toolbar-item code-language", onChange: onCodeLanguageSelect, options: codeLanguges, value: codeLanguage }), jsx("i", { className: "chevron-down inside" })] })) : (jsxs(Fragment, { children: [jsx("button", { onClick: () => {
                            editor.dispatchCommand(FORMAT_TEXT_COMMAND, "bold");
                        }, className: "toolbar-item spaced " + (isBold ? "active" : ""), "aria-label": "Format Bold", children: jsx("i", { className: "format bold" }) }), jsx("button", { onClick: () => {
                            editor.dispatchCommand(FORMAT_TEXT_COMMAND, "italic");
                        }, className: "toolbar-item spaced " + (isItalic ? "active" : ""), "aria-label": "Format Italics", children: jsx("i", { className: "format italic" }) }), jsx("button", { onClick: () => {
                            editor.dispatchCommand(FORMAT_TEXT_COMMAND, "underline");
                        }, className: "toolbar-item spaced " + (isUnderline ? "active" : ""), "aria-label": "Format Underline", children: jsx("i", { className: "format underline" }) }), jsx("button", { onClick: () => {
                            editor.dispatchCommand(FORMAT_TEXT_COMMAND, "strikethrough");
                        }, className: "toolbar-item spaced " + (isStrikethrough ? "active" : ""), "aria-label": "Format Strikethrough", children: jsx("i", { className: "format strikethrough" }) }), jsx("button", { onClick: () => {
                            editor.dispatchCommand(FORMAT_TEXT_COMMAND, "code");
                        }, className: "toolbar-item spaced " + (isCode ? "active" : ""), "aria-label": "Insert Code", children: jsx("i", { className: "format code" }) }), jsx("button", { onClick: insertLink, className: "toolbar-item spaced " + (isLink ? "active" : ""), "aria-label": "Insert Link", children: jsx("i", { className: "format link" }) }), isLink &&
                        createPortal(jsx(FloatingLinkEditor, { editor: editor }), document.body), jsx(Divider, {}), jsx("button", { onClick: () => {
                            editor.dispatchCommand(FORMAT_ELEMENT_COMMAND, "left");
                        }, className: "toolbar-item spaced", "aria-label": "Left Align", children: jsx("i", { className: "format left-align" }) }), jsx("button", { onClick: () => {
                            editor.dispatchCommand(FORMAT_ELEMENT_COMMAND, "center");
                        }, className: "toolbar-item spaced", "aria-label": "Center Align", children: jsx("i", { className: "format center-align" }) }), jsx("button", { onClick: () => {
                            editor.dispatchCommand(FORMAT_ELEMENT_COMMAND, "right");
                        }, className: "toolbar-item spaced", "aria-label": "Right Align", children: jsx("i", { className: "format right-align" }) }), jsx("button", { onClick: () => {
                            editor.dispatchCommand(FORMAT_ELEMENT_COMMAND, "justify");
                        }, className: "toolbar-item", "aria-label": "Justify Align", children: jsx("i", { className: "format justify-align" }) }), jsx(Divider, {}), jsx("button", { onClick: () => {
                            setImgActive(true);
                        }, className: "toolbar-item spaced", "aria-label": "Image", children: jsx("i", { className: "format image" }) }), jsx("button", { onClick: () => {
                            setVideoActive(true);
                        }, className: "toolbar-item spaced", "aria-label": "Video", children: jsx("i", { className: "format video" }) })] })), jsx(Modal, { active: video, setActive: setVideoActive, children: jsx(AddUrl, { title: "Add Video", onCancel: () => setVideoActive(false), onConfirm: (url => {
                        setVideoActive(false);
                        insertVideo(url);
                    }) }) }), jsx(Modal, { className: "w-224", active: imgActive, setActive: setImgActive, children: !!getImages ? jsx(ImageSelection, { uploadUrl: uploadUrl, onCancel: () => {
                        setImgActive(false);
                    }, onSelect: (images) => {
                        images.map(p => insertImage(p));
                        setImgActive(false);
                    }, getFiles: getImages }) : jsx(AddUrl, { title: "Add Image", onCancel: () => setVideoActive(false), onConfirm: (url => {
                        setImgActive(false);
                        insertImage(url);
                    }) }) })] }));
}

export { ToolbarPlugin as default };
